/*******************************************************************************
Calculator Project
Copyright 1997, 1998 R3 Software Ltd. All Rights Reserved.

<file>
$Id: CalcApplication.cpp,v 1.11 1998/12/03 08:29:39 timmy Exp $
Component:	Application
File:		CalcApplication.c
Summary:	Central application code.
Created:	1997.10.01
Author:		timmy
History:
1998.10.20 timmy Removed use of libprefs. Updated code style.
</file>
*******************************************************************************/


#include "CalcApplication.h"


// Standard Includes

#include <stdlib.h>
#include <stdio.h>
#include <string.h>


// BeOS Includes

#ifndef _SCREEN_H
	#include <interface/Screen.h>
#endif

#ifndef _DIRECTORY_H
	#include <storage/Directory.h>
#endif

#ifndef _FIND_DIRECTORY_H
	#include <storage/FindDirectory.h>
#endif

#ifndef _PATH_H
	#include <storage/Path.h>
#endif


// Version Constants

const int32 k_deskCalcVers = 200;
const int32 k_deskCalcMinVers = 200;


// Application Includes

#ifndef _CALC_WINDOW_H
	#include "CalcWindow.h"
#endif


// Constants

const char *PREFS_FILE_NAME = "DeskCalc_settings";
float DEFAULT_WIDTH = 220.0;
float DEFAULT_HEIGHT = 140.0;


// Code!

/*******************************************************************************
<func>
Class:		CalcApplication
Function:	CalcApplication(
				const char *calcDesc,
				const char *appSig,
				const char *calcName);
Summary:	Constructor.
Params:		calcDesc	calculator description
			appSig		application signature
			calcName	calculator name text
Author:		timmy
Created:	1997.10.01
History:
</func>
*******************************************************************************/
CalcApplication::CalcApplication(
		const char *calcDesc,
		const char *appSig,
		const char *calcName
		)
	:	BApplication(appSig),
		mCalcWindow(NULL)
		{
	
	// create new window or load from prefs if available
	if (!LoadPrefs()){
		// allocate a new (default) calculator window
		puts("No settings available: create as default.\n");
		float x, y;
		{	// center on screen
			BScreen screen;
			x = (screen.Frame().Width() + 1 - DEFAULT_WIDTH) * 0.5;
			y = (screen.Frame().Height() + 1 - DEFAULT_HEIGHT) * 0.5;
			} // end block
		BRect frame(x, y, x + DEFAULT_WIDTH, y + DEFAULT_HEIGHT);
		mCalcWindow = new CalcWindow(frame, calcName, calcDesc);
		} // end if
	
	// reveal window
	mCalcWindow->Show();
	
	// move window to current workspace
	mCalcWindow->SetWorkspaces(B_CURRENT_WORKSPACE);
	
} // end CalcApplication::CalcApplication()


/*******************************************************************************
<func>
Class:		CalcApplication
Function:	virtual ~CalcApplication();
Summary:	Destructor.
Author:		timmy
Created:	1997.10.01
History:
1997.10.13 timmy Added code to archive the current calculator state.
</func>
*******************************************************************************/
CalcApplication::~CalcApplication(){

	// save current user preferences
	StorePrefs();

} // end CalcApplication::~CalcApplication()


/*******************************************************************************
<func>
Class:		CalcApplication
Function:	virtual void ReadyToRun();
Summary:	Application boot completed hook.
Author:		timmy
Created:	1997.10.01
History:
</func>
*******************************************************************************/
void CalcApplication::ReadyToRun(){
	
	// TODO: implement me!

} // end CalcApplication::ReadyToRun()


/*******************************************************************************
<func>
Class:		CalcApplication
Function:	virtual void AboutRequested();
Summary:	Application about box request hook.
Author:		timmy
Created:	1997.10.01
Notes:		-
History:
</func>
*******************************************************************************/
void CalcApplication::AboutRequested(){
	
	// TODO: implement me!

} // end CalcApplication::AboutRequested()


/*******************************************************************************
<func>
Class:		CalcApplication
Function:	virtual bool QuitRequested();
Summary:	Application quit request hook.
Returns:	Are we ready to quit? (always true)
Author:		timmy
Created:	1997.10.01
History:
</func>
*******************************************************************************/
bool CalcApplication::QuitRequested(){
	
	return true;

} // end CalcApplication::QuitRequested()


/*******************************************************************************
<func>
Class:		CalcApplication
Function:	bool LoadPrefs();
Summary:	Load preferences data.
Returns:	true if successful load, else false
Author:		timmy
Created:	1997.10.14
Notes:		Preferences are store as a flattened message in a BFile.
History:
1998.11.05 timmy Removed libprefs code.
</func>
*******************************************************************************/
bool CalcApplication::LoadPrefs(){

	int32 prefsCalcVers;
	BMessage archive;
	status_t status;
	BMessage windowArchive;

	// locate preferences file
	BFile *prefsFile = FindPrefsFile(false);
	if (!prefsFile){
		puts("no preference file found.");
		return false;
		} // end if
	
	// unflatten data
	if ((status = archive.Unflatten(prefsFile)) < B_OK){
		goto LoadPrefs_exit;
		} // end if

	// check stored prefs version
	if (archive.FindInt32("vers", &prefsCalcVers) < B_OK){
		fprintf(stderr, "ERROR: No preferences version! bad....\n");
		return false;
		} // end if
	if (prefsCalcVers < k_deskCalcMinVers){	
		fprintf(stderr, "Preferences too old!\n");
		return false;
		} // end else
	if (prefsCalcVers > k_deskCalcVers){	
		fprintf(stderr, "Preferences too new!\n");
		return false;
		} // end else

	// instantiate CalcWindow
	archive.FindMessage("window", &windowArchive);
	if (!validate_instantiation(&windowArchive, "CalcWindow")){
		fprintf(stderr, "Failed validate_instantiation().\n");
		return false;
		} // end if
	mCalcWindow = CalcWindow::Instantiate(&windowArchive);
	if (mCalcWindow == NULL){
		fprintf(stderr, "Bad perferences data.\n");
		return false;
		} // end if
	
	// check out window
	if (mCalcWindow->CountChildren() < 1){
		fprintf(stderr, "Bad CalcWindow perferences data.\n");
		delete mCalcWindow;
		return false;
		} // end if
	
	// move window to the current workspace
	mCalcWindow->SetWorkspaces(B_CURRENT_WORKSPACE);
	
	// cool!
	puts("Loaded preferences.\n");
	
	// all was well!
	delete prefsFile;
	return true;
	
LoadPrefs_exit:
	
	delete prefsFile;
	return false;
	
} // end CalcApplication::LoadPrefs()


/*******************************************************************************
<func>
Class:		CalcApplication
Function:	void StorePrefs();
Summary:	Updates the perferences data.
Author:		timmy
Created:	1997.10.14
Notes:		-
History:
1998.10.28 timmy Rewrote to use simple file based model.
1998.12.03 timmy Made debug conditional on NDEBUG.
</func>
*******************************************************************************/
void CalcApplication::StorePrefs(){

	BMessage archive;

	// archive the current state of the calculator
	mCalcWindow->Lock();
	BMessage windowArchive;
	if (mCalcWindow->Archive(&windowArchive, true) < B_OK){
		fprintf(stderr, "bummer! error when we tried to archive\n");
		return;
		} // end if
	#if !defined(NDEBUG)
	windowArchive.PrintToStream();
	#endif
	mCalcWindow->Unlock();

	// archive window and app version
	archive.AddMessage("window", &windowArchive);
	archive.AddInt32("vers", k_deskCalcVers);
	#if !defined(NDEBUG)
	archive.PrintToStream();
	#endif
	
	// flatten entire acrhive and write to settings file
	BFile *prefsFile = FindPrefsFile(true);
	if (prefsFile){
		if (archive.Flatten(prefsFile) < B_OK){
			fprintf(stderr, "bummer! error when we tried to flatten\n");
			return;
			} // end if
		delete prefsFile;
		} // end if
	else fprintf(stderr, "ERROR: could not store prefs.\n");
	
} // end CalcApplication::StorePrefs()


/*******************************************************************************
<func>
Class:		CalcApplication
Function:	BFile *FindPrefsFile(bool write);
Summary:	Finds the appropriate preferences file.
Param:		write		open for writing
Author:		timmy
Created:	1998.10.28
Notes:
1998.12.03 timmy Made debug conditional on NDEBUG.
</func>
*******************************************************************************/
BFile *CalcApplication::FindPrefsFile(bool write){

	status_t err;

	// find user settings directory	
	BPath prefsPath;
	if (find_directory(B_USER_SETTINGS_DIRECTORY, &prefsPath) < B_OK){
		fprintf(stderr, "ERROR: could not find user settings directory!\n");
		return NULL;
		} // end if

	// create directory from path
	#if !defined(NDEBUG)
	printf("User settings directory:  %s\n", prefsPath.Path());
	#endif
	BDirectory prefsDir(prefsPath.Path());
	
	// (re)create file if write mode
	if (write){
		BFile *prefsFile = new BFile();
		if (prefsDir.CreateFile(PREFS_FILE_NAME, prefsFile) < B_OK){
			fprintf(stderr, "ERROR: could create preferences file!\n");
			delete prefsFile;
			return NULL;
			} // end if
		else return prefsFile;
		} // end if
	
	// otherwise, try to find an existing file
	else {
		BEntry entry;
		if ((err = prefsDir.FindEntry(PREFS_FILE_NAME, &entry)) == B_OK){
			// we found it!
			BFile *prefsFile = new BFile(&entry, B_READ_ONLY);
			return prefsFile;
			} // end if
		else if (err == B_ENTRY_NOT_FOUND){
			// file wasn't there...
			fprintf(stderr, "preferences file not found.\n");
			return NULL;
			} // end else if
		else {
			// misc file error
			fprintf(stderr, "ERROR: file error!\n");
			return NULL;
			} // end else
		} // end else
	
} // end FindPrefsFile()


/******************************************************************************/

// end CalcApplication.cpp